import os

from PyQt5.QtCore import QTranslator, QSettings, Qt
from qgis.PyQt import QtWidgets, QtGui
from qgis.PyQt.QtCore import QCoreApplication

from .resources import *  # 初始化资源，没有这句，图标不会显示

from .cyanlove_import_sqlite import cyanlove_import_sqlite
from .cyanlove_customdraw import cyanlove_customdraw
from .cyanlove_createpoint import cyanlove_createpoint
from .cyanlove_import_geometry import cyanlove_import_geometry
from .cyanlove_exportwkt import cyanlove_exportwkt
from .cyanlove_sqliteset import cyanlove_sqliteset
from .cyanlove_dockwidget import cyanloveDockWidget


class cyanlove:

    def __init__(self, iface):
        self.iface = iface
        self.plugin_dir = os.path.dirname(__file__)
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'cyanlove_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)
            QCoreApplication.installTranslator(self.translator)

        self.menu = None  # 用于存储主菜单
        self.actions = []  # 用于存储添加的动作

        self.dockwidget_show_shange_neigth = None
        self.dockwidget_export_wkt_tocsv = None
        self.dockwidget_sqlite = None
        self.dockwidget_import_csv_excel_tosqlite = None
        self.dockwidget_draw_polygon = None
        self.dockwidget_custom_draw_polygon = None
        self.dockwidget_draw_point = None

    def cyanlove_add_submenu(self, submenu):
        if self.menu is not None:
            self.menu.addMenu(submenu)  # 将子菜单添加到主菜单
        else:
            self.iface.addPluginToMenu("&申少军工具箱", submenu.menuAction())

    def initGui(self):
        # 创建主菜单 "申少军工具箱"
        self.menu = QtWidgets.QMenu(QCoreApplication.translate("cyanlove", "申少军工具箱"))
        self.iface.mainWindow().menuBar().addMenu(self.menu)
        icon_path1 = ':/plugins/cyanlove/icon/accessibility-svgrepo-com.svg'
        icon_path2 = ':/plugins/cyanlove/icon/activity-svgrepo-com.svg'
        icon_path3 = ':/plugins/cyanlove/icon/address-card-svgrepo-com.svg'
        # 定义子菜单项
        db_submenu = [
            ("Sqlite数据库配置", self.sub_sqlite, icon_path1),
            ("PostGreSQL数据库配置", self.sub_analysis2, icon_path2)
        ]
        export_submenu = [
            ("边界经纬度集导出", self.sub_export_wkt_tocsv, icon_path1),
            ("属性表信息导出", self.sub_analysis2, icon_path2)
        ]
        import_submenu = [
            ("小区表/邻区表CSV/Excel导入(Sqlite)", self.sub_import_csv_excel_tosqlite, icon_path1),
            ("其他文件CSV/Excel导入(Sqlite)", self.sub_analysis2, icon_path2),
            ("栅格数据导入(PostGreSQL)", self.sub_analysis2, icon_path1),
        ]
        create_polygon_submenu = [
            ("创建点图层", self.sub_draw_point, icon_path1),
            ("创建线图层", self.sub_analysis2, icon_path2),
            ("创建面图层", self.sub_draw_polygon, icon_path1),
            ("自定义点、线、面图层=>(逗号,分号)或WKT格式", self.sub_custom_draw_polygon, icon_path1),
        ]
        wangluoyouhua_submenu = [
            ("栅格/邻区显示", self.sub_show_shange_neigth, icon_path1),
            ("坐标定位", self.sub_analysis2, icon_path1),
            ("小区/基站搜索定位", self.sub_analysis2, icon_path1),
        ]
        diliwenxi_submenu = [
            ("缓冲区分析", self.sub_analysis2, icon_path1),
            ("点在边界内", self.sub_analysis2, icon_path1),
            ("边界是否相交", self.sub_analysis2, icon_path1),
            ("面积计算", self.sub_analysis2, icon_path1),
            ("相交合并", self.sub_analysis2, icon_path1),
            ("不相交分离", self.sub_analysis2, icon_path1),
        ]
        # 添加菜单项和子菜单
        self.add_menu_actions("数据库配置", db_submenu)
        self.add_menu_actions("图层数据导出", export_submenu)
        self.add_menu_actions("文件导入", import_submenu)
        self.add_menu_actions("创建图层", create_polygon_submenu)
        self.add_menu_actions("网络优化", wangluoyouhua_submenu)
        self.add_menu_actions("地理化处理分析", diliwenxi_submenu)

    def add_menu_actions(self, menu_name, submenu_items):
        """添加菜单项及其子菜单"""
        # 创建菜单项
        action = QtWidgets.QAction(QCoreApplication.translate("cyanlove", menu_name),
                                   self.iface.mainWindow())
        action.triggered.connect(self.run_analysis)  # 连接到主菜单方法
        self.menu.addAction(action)
        self.actions.append(action)  # 保存动作

        # 创建子菜单
        submenu = self.create_submenu(menu_name, submenu_items)
        action.setMenu(submenu)

    def create_submenu(self, submenu_name, actions):
        """创建子菜单的通用函数"""
        submenu = QtWidgets.QMenu(QCoreApplication.translate("cyanlove", submenu_name), self.iface.mainWindow())
        for action_name, method, icon_path in actions:
            self.add_submenu_action(submenu, action_name, method, icon_path)
        return submenu

    def add_submenu_action(self, submenu, action_name, method, icon_path):
        """添加一个子菜单项的通用函数，带图标"""
        action = QtWidgets.QAction(QCoreApplication.translate("cyanlove", action_name), self.iface.mainWindow())
        action.triggered.connect(method)  # 连接到指定的方法
        action.setIcon(QtGui.QIcon(icon_path))  # 添加图标
        submenu.addAction(action)  # 将动作添加到子菜单

    # 栅格/邻区显示
    def sub_show_shange_neigth(self):
        # 这里添加子菜单分析1的逻辑
        if self.dockwidget_show_shange_neigth is None:
            self.dockwidget_show_shange_neigth = cyanloveDockWidget()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_show_shange_neigth)
            self.dockwidget_show_shange_neigth.setGeometry(1230, 530, 270, 150)  # 设置初始位置和大小

        # 每次点击时，显示 dockwidget
        self.dockwidget_show_shange_neigth.show()
        # 设置为浮动
        self.dockwidget_show_shange_neigth.setFloating(True)

    # 导出WKT格式边界集
    def sub_export_wkt_tocsv(self):
        if self.dockwidget_export_wkt_tocsv is None:
            self.dockwidget_export_wkt_tocsv = cyanlove_exportwkt()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_export_wkt_tocsv)
            self.dockwidget_export_wkt_tocsv.setGeometry(500, 400, 600, 160)

        # 每次点击时，显示 dockwidget
        self.dockwidget_export_wkt_tocsv.show()
        # 设置为浮动
        self.dockwidget_export_wkt_tocsv.setFloating(True)

    # sqlite数据库配置
    def sub_sqlite(self):
        if self.dockwidget_sqlite is None:
            self.dockwidget_sqlite = cyanlove_sqliteset()
            self.iface.addDockWidget(Qt.AllDockWidgetAreas, self.dockwidget_sqlite)
        # 每次点击时，显示 dockwidget
        self.dockwidget_sqlite.show()

    # 导入csv/excel到sqlite数据库
    def sub_import_csv_excel_tosqlite(self):
        if self.dockwidget_import_csv_excel_tosqlite is None:
            self.dockwidget_import_csv_excel_tosqlite = cyanlove_import_sqlite()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_import_csv_excel_tosqlite)
            self.dockwidget_import_csv_excel_tosqlite.setGeometry(400, 300, 700, 180)
        # 每次点击时，显示 dockwidget
        self.dockwidget_import_csv_excel_tosqlite.show()
        # 设置为浮动
        self.dockwidget_import_csv_excel_tosqlite.setFloating(True)

    # 选择csv/excel文件绘制多边形面
    def sub_draw_polygon(self):
        if self.dockwidget_draw_polygon is None:
            self.dockwidget_draw_polygon = cyanlove_import_geometry()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_draw_polygon)
            self.dockwidget_draw_polygon.setGeometry(400, 300, 880, 240)
            # 每次点击时，显示 dockwidget
        self.dockwidget_draw_polygon.show()
        # 设置为浮动
        self.dockwidget_draw_polygon.setFloating(True)

    # 输入经纬度集绘制点，线，面
    def sub_custom_draw_polygon(self):
        if self.dockwidget_custom_draw_polygon is None:
            self.dockwidget_custom_draw_polygon = cyanlove_customdraw()
            self.iface.addDockWidget(Qt.RightDockWidgetArea, self.dockwidget_custom_draw_polygon)
            # 每次点击时，显示 dockwidget
        self.dockwidget_custom_draw_polygon.show()

    # 选择点文件，创建点图层
    def sub_draw_point(self):
        if self.dockwidget_draw_point is None:
            self.dockwidget_draw_point = cyanlove_createpoint()
            self.iface.addDockWidget(Qt.LeftDockWidgetArea, self.dockwidget_draw_point)
            self.dockwidget_draw_point.setGeometry(400, 300, 750, 140)
            # 每次点击时，显示 dockwidget
        self.dockwidget_draw_point.show()
        # 设置为浮动
        self.dockwidget_draw_point.setFloating(True)

    def sub_analysis2(self):
        # 这里添加子菜单分析2的逻辑
        print("执行子菜单分析2...")

    def run_analysis(self):
        # 这里添加栅格分析逻辑
        print("执行栅格分析...")

    def export_wkt(self):
        # 这里添加导出 WKT 逻辑
        print("导出图层边界 WKT...")

    def unload(self):
        # 清除菜单和动作
        if self.menu:
            self.iface.mainWindow().menuBar().removeAction(self.menu.menuAction())
            self.menu.deleteLater()  # 释放菜单资源

        # 清除所有动作
        for action in self.actions:
            action.setEnabled(False)  # 禁用动作，以免出现错误
            self.menu.removeAction(action)  # 从菜单中移除动作
        self.actions.clear()  # 清空动作列表

